#include <config.h>
#include <iostream>

#include "raptheader.h"

using namespace std;

/*
 * Beginnings of C++ wrapper for RPM header and data.
 * Having a rpm version independent ways to access header data makes
 * things a whole lot saner in other places while permitting newer
 * rpm interfaces to be used where supported.
 *
 * Lots to do still:
 * - actually implement "raw" string mode + handle i18n strings correctly
 * - we'll need putTag() method too for genpkglist
 * - for older rpm versions we'll need to use one of rpmHeaderGetEntry(),
 *   headerGetEntry() or headerGetRawEntry() depending on tag and content
 * - we'll want to eventually replace rpmhandler's HeaderP with raptHeader(),
 *   add constructors for reading in from files, db iterators & all etc...
 */

raptHeader::raptHeader(Header h)
{
   Hdr = headerLink(h);
}

raptHeader::~raptHeader()
{
   headerUnlink(Hdr);
}

bool raptHeader::hasTag(rpmTag tag)
{
   return headerIsEntry(Hdr, tag);
}

// XXX: should have a way to pass back formatting error messages
string raptHeader::format(const string fmt)
{
   string res = "";
   char *s;
   s = headerSprintf(Hdr, fmt.c_str(), rpmTagTable, rpmHeaderFormats, NULL);
   if (s) {
      res = string(s);
      free(s);
   }
   return res;
}

bool raptHeader::getTag(rpmTag tag, uint32_t &data)
{
   vector<uint32_t> _data;
   bool ret = false;

   if (getTag(tag, _data) && _data.size() == 1) {
      data = _data[0];
      ret = true;
   }
   return ret;
}

bool raptHeader::getTag(rpmTag tag, string &data, bool raw)
{
   vector<string> _data;
   bool ret = false;

   if (getTag(tag, _data, raw) && _data.size() == 1) {
      data = _data[0];
      ret = true;
   }
   return ret;
}

bool raptHeader::getTag(rpmTag tag, vector<string> &data, bool raw)
{
   bool ret = false;
   void *val = NULL;
   int rc = 0;
   rpmTagCount count = 0;
   rpmTagType type = (rpmTagType)0;
   HE_t he = (HE_t)memset(alloca(sizeof(*he)), 0, sizeof(*he));
   he->tag = (rpmTag)tag;

   if (tag == RPMTAG_OLDFILENAMES) {
      rc = headerGet(Hdr, he, 0);
      if (rc) {
	 type = he->t;
	 val = he->p.ptr;
	 count = he->c;
      }
   } else if (raw) {
	he->t = type;
	he->p.str = (const char*)val;
	he->c = count;
	rc = headerGet(Hdr, he, tag);
	if (rc) {
	   type = he->t;
	   val = he->p.ptr;
	   count = he->c;
	}
   } else {
      rc = headerGet(Hdr, he, 0);
      if (rc) {
	 type = he->t;
	 val = he->p.ptr;
	 count = he->c;
      }
   }

   if (rc) {
      switch (type) {
	 case RPM_STRING_TYPE: {
	    char *hdata = (char *)val;
	    data.push_back(hdata);
	    ret = true;
	    break;
	 }
	 case RPM_STRING_ARRAY_TYPE:
	 case RPM_I18NSTRING_TYPE: {
	    char **hdata = (char **)val;
	    for (int i = 0; i < count; i++) {
	       data.push_back(hdata[i]);
	    }
	    free(hdata);
	    ret = true;
	    break;
	 }
	 default: 
	    break;
      }
   }
   return ret;
}

bool raptHeader::getTag(rpmTag tag, vector<uint32_t> &data)
{
   bool ret = false;
   rpmTagCount count = 0;
   rpmTagType type = (rpmTagType)0;
   HE_t he = (HE_t)memset(alloca(sizeof(*he)), 0, sizeof(*he));

   he->tag = tag;

   if (headerGet(Hdr, he, 0)) {
      if (type == RPM_UINT32_TYPE) {
	 uint32_t *hdata = he->p.ui32p;
	 for (int i = 0; i < count; i++) {
	    data.push_back(hdata[i]);
	 }
	 ret = true;
      }
   }
   return ret;
}

// vim:sts=3:sw=3
